import * as types from './.dict.js';
import config from '@frame/config';
const { pageSize = 10 } = config;

const api = {
  types: '/gaf/code/types',
  items: '/gaf/code/items',
  item: id => `/gaf/code/items/${id}`,
};

// initial state
export const state = () => ({
  types: [], // 字典分类
  items: [], // 字典项
  type: null, // 当前字典分类
  current: null, // 当前字典项
  total: 0, // 字典项总数
});

// actions
export const actions = {
  async loadType({ commit }, payload = {}) {
    const res = await this.$axios.$get(api.types);
    if (res.errcode === undefined || res.errcode === 0) {
      console.log(res);
      commit(types.TYPE_LOADED, res.data);
    }
    return res;
  },
  async createType({ commit, state }, payload) {
    const { code, name, alias } = payload;
    const res = await this.$axios.$post(api.types, { code, name, alias });
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.TYPE_CREATED, res);
    }
    return res;
  },
  async deleteType({ commit, state }, { id }) {
    const res = await this.$axios.$delete(api.item(id));
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.TYPE_DELETED, { id });
    }
    return res;
  },
  async updateType({ commit, state }, payload = {}) {
    const { id, name, alias } = payload;
    const res = await this.$axios.$post(api.item(id), { name, alias });
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.TYPE_UPDATED, res);
    }
    return res;
  },
  async loadItem({ state, commit }, payload = {}) {
    let { type, page = 1, size = pageSize } = payload;
    if (!type) type = state.type.code;
    const skip = Math.max(0, (page - 1) * size);
    const res = await this.$axios.$get(`${api.items}?type=${type}&skip=${skip}&limit=${size}`);
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.ITEM_LOADED, res);
    }
    return res;
  },
  async createItem({ commit, state }, payload) {
    const { type, code, name, alias, status } = payload;
    const res = await this.$axios.$post(api.items, { code, name, alias, status }, { params: { type } });
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.ITEM_CREATED, res);
    }
    return res;
  },
  async deleteItem({ commit, state }, { id }) {
    const res = await this.$axios.$delete(api.item(id));
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.ITEM_DELETED, { id });
    }
    return res;
  },
  async updateItem({ commit, state }, payload = {}) {
    const { id, code, name, alias, status } = payload;
    const res = await this.$axios.$post(api.item(id), { name, alias, status });
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.ITEM_UPDATED, res);
    }
    return res;
  },
  async selectType({ commit, dispatch, state }, payload) {
    commit(types.TYPE_SELECTED, payload);
    await dispatch('loadItem', { type: payload.code });
  },
};

// mutations
export const mutations = {
  [types.TYPE_LOADED](state, payload) {
    state.types = payload;
  },
  [types.TYPE_SELECTED](state, payload) {
    state.type = payload;
  },
  [types.TYPE_CREATED](state, payload) {
    state.types.push(payload);
  },
  [types.TYPE_DELETED](state, payload) {
    const idx = state.types.findIndex(p => p.id === payload.id);
    state.types.splice(idx, 1);
    if (state.type && state.type.id === payload.id) {
      state.type = null;
      state.items = [];
    }
  },
  [types.TYPE_UPDATED](state, payload) {
    const idx = state.types.findIndex(p => p.id == payload.id);
    state.types.splice(idx, 1, payload);
    if (state.type && state.type.id === payload.id) {
      state.type = payload;
    }
  },
  [types.ITEM_LOADED](state, { data, total }) {
    state.items = data;
    state.total = total;
  },
  [types.ITEM_CREATED](state, payload) {
    state.items.push(payload);
    state.total += 1;
    state.current = payload;
  },
  [types.ITEM_DELETED](state, payload) {
    const idx = state.items.findIndex(p => p.id === payload.id);
    state.items.splice(idx, 1);
    state.total -= 1;
  },
  [types.ITEM_UPDATED](state, payload) {
    const idx = state.items.findIndex(p => p.id === payload.id);
    state.items.splice(idx, 1, payload);
    state.current = payload;
  },
  [types.ITEM_SELECTED](state, payload) {
    state.current = payload;
  },
};
