import assert from 'assert';
import * as types from '../.mutation.js';
import _ from 'lodash';
import config from '@frame/config';
const { pageSize = 10 } = config;

const api = {
  users: '/gaf/users',
  deptUsers: '/gaf/bff/dept/users',
  roleUsers: '/gaf/bff/role/users',
  bindDept: '/gaf/bind/user-dept/items',
  bindRole: '/gaf/bind/user-role/items',
};

// initial state
// shape: [{ id, quantity }]
export const state = () => ({
  items: [], //所有数据
  current: null,
  total: 0,
  deptIds: [], // 当前用户的部门ID列表
  roleIds: [], // 当前用户的角色ID列表
});

// actions
export const actions = {
  async query({ commit }, { deptId, roleId, filter = {}, skip = 0, limit = pageSize }) {
    let res;
    if (deptId !== undefined && deptId !== null) {
      res = await this.$axios.$get(`${api.deptUsers}`, { params: { deptId, ...filter, skip, limit } });
    } else if (roleId !== undefined && roleId !== null) {
      res = await this.$axios.$get(`${api.roleUsers}`, { params: { roleId, ...filter, skip, limit } });
    } else {
      res = await this.$axios.$get(`${api.users}`, { params: { ...filter, skip, limit } });
    }
    return res;
  },
  async load({ commit, dispatch }, { deptId, roleId, filter = {}, paging = {} }) {
    const { page = 1, size = pageSize } = paging;
    const skip = Math.max(0, (page - 1) * size);
    const res = await dispatch('query', { deptId, roleId, filter, skip, limit: size });
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.LOADED, res);
    }
  },
  async create({ commit, state }, payload) {
    const { account, password: secret } = payload;
    const user = _.omit(payload, 'account', 'password', 'confirm');
    let res = await this.$axios.$post(`${api.users}`, { ...user, account: { account, secret } });
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.CREATED, res);
    }
    return res;
  },
  async delete({ commit, state }, { userId }) {
    const res = await this.$axios.$delete(`${api.users}/${userId}`);
    if (res.errcode === undefined || res.errcode === 0) commit(types.DELETED, { userId });
    return res;
  },
  async update({ commit, state }, payload = {}) {
    const { userId } = payload;
    const data = _.omit(payload, ['userId']);
    const res = await this.$axios.$post(`${api.users}/${userId}`, data);
    if (res.errcode === undefined || res.errcode === 0) commit(types.UPDATED, res);
    return res;
  },
  async passwd({ commit, state }, payload = {}) {
    const { userId, account, password: secret } = payload;
    const res = await this.$axios.$post(`${api.users}/${userId}/acct`, { account, secret });
    return res;
  },
  async bindDepts({ commit, state }, { deptIds, userId, replace = true } = {}) {
    // 同时绑定多个部门
    let res;
    if (deptIds && deptIds.length > 0) {
      res = await this.$axios.$post(`${api.bindDept}/batch`, { targets: deptIds, source: userId }, { params: { replace, userId } });
    } else if (replace) {
      // TODO: 删除所有部门
      res = await this.$axios.$delete(api.bindDept, { source: userId });
    }
    if (res.errcode === undefined || res.errcode === 0) {
      // 刷新用户部门列表
      commit('deptIds', deptIds || []);
    }
    return res;
  },
  async bindRoles({ commit, state }, { roleIds, userId, replace = true } = {}) {
    // 同时绑定多个角色
    let res;
    if (roleIds && roleIds.length > 0) {
      res = await this.$axios.$post(`${api.bindRole}/batch`, { targets: roleIds, source: userId }, { params: { replace, userId } });
    } else if (replace) {
      // TODO: 删除所有角色
      res = await this.$axios.$delete(api.bindRole, { source: userId });
    }
    if (res.errcode === undefined || res.errcode === 0) {
      // 刷新用户角色列表
      commit('roleIds', roleIds || []);
    }
    return res;
  },
  async fetchDepts({ commit, state }, { userId } = {}) {
    const res = await this.$axios.$get(`${api.bindDept}`, { source: userId });
    let ids = [];
    if (res.errcode && res.errcode !== 0) {
      console.warn('user.fetchDepts error:', res);
    } else {
      ids = res.data.map(p => p.target);
    }
    commit('deptIds', ids);
    return ids;
  },
  async fetchRoles({ commit, state }, { userId } = {}) {
    const res = await this.$axios.$get(`${api.bindRole}`, { source: userId });
    let ids = [];
    if (res.errcode && res.errcode !== 0) {
      console.warn('user.fetchRoles error:', res);
    } else {
      ids = res.data.map(p => p.target);
    }
    commit('roleIds', ids);
    return ids;
  },
};

// mutations
export const mutations = {
  [types.LOADED](state, payload = {}) {
    ({ data: state.items, total: state.total } = payload);
  },
  [types.SELECTED](state, payload) {
    state.current = payload;
  },
  [types.CREATED](state, payload) {
    state.items.push(payload);
    state.current = payload;
    state.total += 1;
  },
  [types.DELETED](state, payload) {
    const idx = state.items.findIndex(p => p.userId === payload.userId);
    state.items.splice(idx, 1);
    state.total -= 1;
  },
  [types.UPDATED](state, payload) {
    const idx = state.items.findIndex(p => p.userId === payload.userId);
    state.items.splice(idx, 1, payload);
    state.current = payload;
  },
  roleIds(state, payload) {
    state.roleIds = payload;
  },
  deptIds(state, payload) {
    state.deptIds = payload;
  },
};
