import assert from 'assert';
import * as types from '../.types.js';
import config from '@frame/config';

const baseUrl = '/xms/user/accts';
const queryAppnRole = 'xms/bff/acct/roles';
const RelationAdd = 'xms/bind/app-acct/items';
const acctrole = 'xms/bind/acct-role/items';

const { pageSize = 10 } = config;
const api = {
  accts: '/xms/user/accts',
  apps: '/xms/bff/acct/apps',
  appAccts: '/xms/bff/app/accts',
  roles: '/xms/bff/acct/roles',
  // 应用帐号绑定关系 source appId  target acctId
  appBind: '/xms/bind/app-acct/items',
  // 帐号角色绑定关系 source acctId  target roleId
  roleBind: '/xms/bind/acct-role/items',
  orgAccts: '/xms/bff/org/accts',
};

// 参数帐号id  appid
// initial state
export const state = () => ({
  items: [], //所有数据
  list: [], //指定用户帐号数据
  roles: [], //指定应用帐号的角色数据
  apps: [], //指定帐号应用绑定关系
  current: null,
  total: 0, // 字典项总数
});

// actions
export const actions = {
  async load({ commit }, payload) {
    let { page = 1, size = pageSize } = payload.paging || payload;
    const skip = Math.max(0, (page - 1) * size);
    const res = await this.$axios.$get(`${api.accts}?skip=${skip}&limit=${pageSize}`, { params: { ...payload.filter } });
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.LOADED, res);
    }
    return res;
  },
  async list({ commit }, { userId }) {
    if (!userId) {
      commit(types.LISTED, { data: [] });
      return [];
    }
    const res = await this.$axios.$get(`${api.accts}`, { userId });
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.LISTED, res);
    }
    return res;
  },
  async create({ commit, state }, payload) {
    let userId = payload.userId;
    const res = await this.$axios.$post(`${api.accts}`, payload, { params: { userId } });
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.CREATED, res);
    }
    return res;
  },
  async delete({ commit, state }, { userId, acctId }) {
    const res = await this.$axios.$delete(`${api.accts}/${acctId}`, { userId });
    return res;
  },
  // 需要添加主账号标识字段
  async update({ commit, state }, payload) {
    const { acctId, status, secret, expired, remark, loginMode, master } = payload;
    const res = await this.$axios.$post(`${baseUrl}/${acctId}`, { status, secret, expired, remark, loginMode, master });
    if (res.errcode === undefined || res.errcode === 0) {
      if (payload.user) res.user = payload.user;
      commit(types.UPDATED, res);
    }
    return res;
  },
  async fetch({ commit }, { userId, acctId } = {}) {
    if (!acctId) {
      commit(types.DETAILS, { userId });
      return { userId };
    }
    // fetch acct info
    const res = await this.$axios.$get(`${api.accts}/${acctId}`);
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.DETAILS, res);
    }
    return res;
  },
  async fetchAppAccts({ commit }, { filter = {}, paging = {}, appId } = {}) {
    let { page = 1, size = pageSize } = paging;
    const skip = Math.max(0, (page - 1) * size);
    const res = await this.$axios.$get(`${api.appAccts}`, { params: { ...filter, appId, skip, limit: size } });
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.LOADED, res);
    }
    return res;
  },
  async fetchOrgAccts({ commit }, { filter = {}, paging = {}, orgId } = {}) {
    let { page = 1, size = pageSize } = paging;
    const skip = Math.max(0, (page - 1) * size);
    const res = await this.$axios.$get(`${api.orgAccts}`, { params: { ...filter, orgId, skip, limit: size } });
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.LOADED, res);
    }
    return res;
  },
  async fetchAcctApps({ commit, dispatch }, { acctId }) {
    if (!acctId) {
      commit('apps', []); // clear
      return;
    }
    const res = await this.$axios.$get(`${api.apps}`, { acctId });
    if (res.errcode === undefined || res.errcode === 0) {
      commit('apps', res.data);
    }
    return res;
  },

  async fetchAcctRoles({ commit, dispatch }, { acctId, appId }) {
    if (!acctId) {
      commit('roles', []);
      return;
    }
    const res = await this.$axios.$get(`${api.roleBind}`, { source: acctId, appId });
    if (res.errcode === undefined || res.errcode === 0) {
      commit('roles', res.data);
    }
    return res;
  },
  async bindApp({ commit, state, dispatch }, { userId, appId, acctId, mode = [], bindId } = {}) {
    assert(userId && appId && acctId, 'userId、appId、acctId均不能为空');
    let res;
    const param = mode.join(',');
    console.log('bindApp:', param, mode);
    if (!bindId) {
      // 添加帐号和应用绑定关系
      res = await this.$axios.$post(`${api.appBind}`, { source: appId, target: acctId, userId, appId, param });
    } else {
      // 修改帐号和应用绑定参数
      res = await this.$axios.$post(`${api.appBind}/${bindId}`, { param });
    }
    if (res.errcode === undefined || res.errcode === 0) {
      // 重新加载帐号app列表
      dispatch('fetchAcctApps', { acctId });
    }
    return res;
  },
  // 添加帐号角色绑定关系
  async bindRoles({ commit, state, dispatch }, { userId, appId, acctId, roles } = {}) {
    const res = await this.$axios.$post(`${api.roleBind}/batch`, { source: acctId, targets: roles, userId, appId });
    return res;
  },
  async deleteRoles({ commit, state }, { id }) {
    // TODO: 循环删除角色
    // ids.every(async id => {
    const res = await this.$axios.$delete(`${api.roleBind}/${id}`);
    if (res.errcode && res.errcode !== 0) {
      console.warn('删除帐号-角色关系失败');
    }
    return res;
    // });
  },
  async deleteApp({ commit, state }, { id, target, acctId, appId, userId }) {
    assert(id && (acctId || target) && appId && userId, 'id && acctId && appId && userId不能为空');
    let res = await this.$axios.$delete(`${api.appBind}/${id}`);
    if (res.errcode && res.errcode !== 0) {
      console.warn('删除帐号-应用关系失败');
      return res;
    }
    commit('deleteApp', { id });
    // 删除帐号中该应用角色
    res = await this.$axios.$delete(`${api.roleBind}`, { userId, appId, source: target || acctId });
    return res;
  },
  async deleteAppAcct({ commit, state }, { target, acctId, appId, userId }) {
    assert((acctId || target) && appId && userId, 'acctId && appId && userId不能为空');
    let res = await this.$axios.$delete(`${api.appBind}`, { source: appId, target: acctId });
    if (res.errcode && res.errcode !== 0) {
      console.warn('删除帐号-应用关系失败');
      return res;
    }
    commit('deleteApp', { acctId });
    // 删除帐号中该应用角色
    return res;
  },
};

// mutations
export const mutations = {
  [types.LOADED](state, { data, total }) {
    state.items = data;
    state.total = total;
  },
  [types.LISTED](state, { data }) {
    state.list = data;
  },
  [types.CREATED](state, payload) {
    state.items.push(payload);
    state.list.push(payload);
    state.current = payload;
  },
  [types.DELETED](state, payload) {
    let idx = state.items.findIndex(p => p.acctId === payload.acctId);
    state.items.splice(idx, 1);
    idx = state.list.findIndex(p => p.acctId === payload.acctId);
    state.list.splice(idx, 1);
  },
  [types.UPDATED](state, payload) {
    let idx = state.items.findIndex(p => p.acctId === payload.acctId);
    state.items.splice(idx, 1, payload);
    idx = state.list.findIndex(p => p.acctId === payload.acctId);
    state.list.splice(idx, 1, payload);
  },
  [types.DETAILS](state, payload) {
    state.current = payload;
  },
  [types.SELECTED](state, payload) {
    state.current = payload;
  },
  roles(state, payload) {
    state.roles = payload;
  },
  apps(state, datas = []) {
    state.apps = datas;
  },
  deleteApp(state, { acctId }) {
    let idx = state.items.findIndex(p => p.acctId === acctId);
    state.items.splice(idx, 1);
    idx = state.list.findIndex(p => p.acctId === acctId);
    state.list.splice(idx, 1);
  },
};
