import * as types from '../.types.js';

const api = {
  items: '/xms/user/groups',
  usergroup: '/xms/bind/group-user/items',
  // 应用帐号绑定关系 source appId  target acctId
  appBind: '/xms/bind/group-app/items',
  // 帐号角色绑定关系 source acctId  target roleId
  roleBind: '/xms/bind/group-role/items',
  apps: '/xms/bff/group/apps',
  roles: '/xms/bff/group/roles',
};

// initial state
export const state = () => ({
  items: [], // 群组列表
  total: 0, // 总数
  current: null,
  apps: [],
  roles: [],
  roleAll: null,
  roletotal: 0,
});

// actions
export const actions = {
  // 创建群组
  async create({ commit, state }, payload) {
    const res = await this.$axios.$post(`${api.items}`, payload);
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.CREATED, res);
    }
    return res;
  },
  // 更新群组
  async update({ commit, state }, payload = {}) {
    const { groupId } = payload;
    const res = await this.$axios.$post(`${api.items}/${groupId}`, payload);
    if (res.errcode === undefined || res.errcode === 0) commit(types.UPDATED, res);
    return res;
  },
  // 删除群组
  async delete({ commit, state }, payload = {}) {
    const { groupId } = payload;
    const res = await this.$axios.$delete(`${api.items}/${groupId}`);
    if (res.errcode === undefined || res.errcode === 0) commit(types.DELETED, { groupId });
    return res;
  },
  // 查询群组列表
  async load({ state, commit }, payload) {
    const res = await this.$axios.$get(`${api.items}`, { limit: 500 });
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.LOADED, res.data);
    }
    return res;
  },
  // 群组和用户实现绑定
  async ugroupBind({ commit, state }, { groupId, userIds, replace = false } = {}) {
    if (userIds.length > 1) {
      return await this.$axios.$post(`${api.usergroup}/batch`, { source: groupId, targets: userIds, userId: userIds[0] }, { params: { replace } });
    } else {
      return await this.$axios.$post(`${api.usergroup}`, { source: groupId, target: userIds[0], userId: userIds[0] });
    }
    // return res;
  },
  // 群组和用户解除绑定
  async ugroupDel({ commit, state }, { id } = {}) {
    const res = await this.$axios.$delete(`${api.usergroup}/` + id);
    return res;
  },
  // 批量接触群组用户绑定
  async batchDelete({ commit, state, dispatch }, { list }) {
    return Promise.all(list.map(e => this.$axios.$delete(`${api.usergroup}/${e}`)));
  },
  // 添加群组应用绑定
  async bindApp({ commit, state, dispatch }, { groupId, appId, mode = [], bindId } = {}) {
    let res;
    const param = mode.join(',');
    if (!bindId) {
      // 添加帐号和应用绑定关系
      res = await this.$axios.$post(`${api.appBind}`, { source: groupId, target: appId, appId, param });
    } else {
      // 修改帐号和应用绑定参数
      res = await this.$axios.$post(`${api.appBind}/${bindId}`, { param });
    }
    if (res.errcode === undefined || res.errcode === 0) {
      // 重新加载帐号app列表
      dispatch('fetchGroupApps', { groupId });
    }
    return res;
  },
  // 删除群组应用绑定
  async deleteGroupApp({ commit, state, dispatch }, { groupId, appId }) {
    let res = await this.$axios.$delete(`${api.appBind}`, { source: groupId, target: appId });
    if (res.errcode && res.errcode !== 0) {
      console.warn('删除帐号-应用关系失败');
      return res;
    }
    if (res.errcode === undefined || res.errcode === 0) {
      // 重新加载群组app列表
      dispatch('fetchGroupApps', { groupId });
    }
    // 删除群组中该应用角色
    res = await this.$axios.$delete(`${api.roleBind}`, { appId, source: groupId });
    return res;
  },
  // 添加群组角色绑定关系
  async bindRoles({ commit, state, dispatch }, { appId, groupId, roles } = {}) {
    const res = await this.$axios.$post(`${api.roleBind}/batch`, { source: groupId, targets: roles, appId });
    return res;
  },
  // 删除角色
  async deleteRoles({ commit, state }, { ids = [] }) {
    return Promise.all(ids.map(e => this.$axios.$delete(`${api.roleBind}/${e}`)));
  },
  // 查询群组绑定的应用
  async fetchGroupApps({ commit, dispatch }, { groupId }) {
    if (!groupId) {
      commit('apps', []); // clear
      return;
    }
    const res = await this.$axios.$get(`${api.apps}`, { groupId });
    if (res.errcode === undefined || res.errcode === 0) {
      commit('apps', res.data);
    }
    return res;
  },
  // 查询群组应用  角色
  async fetchGroupRoles({ commit, dispatch }, { groupId, appId }) {
    if (!groupId || !appId) {
      commit('roles', []); // clear
      return;
    }
    const res = await this.$axios.$get(`${api.roleBind}`, { source: groupId, appId });
    if (res.errcode === undefined || res.errcode === 0) {
      commit('roles', res.data);
    }
    return res;
  },
  // 查询群组所有  角色
  async groupRoles({ commit, dispatch }, { groupId }) {
    if (!groupId) {
      commit('groupRoles', []); // clear
      return;
    }
    const res = await this.$axios.$get(`${api.roleBind}`, { source: groupId });
    if (res.errcode === undefined || res.errcode === 0) {
      commit('groupRoles', res);
    }
    return res;
  },
};

// mutations
export const mutations = {
  [types.LOADED](state, payload) {
    state.items = payload;
  },
  [types.SELECTED](state, payload) {
    state.current = payload;
  },
  [types.CREATED](state, payload) {
    state.items.push(payload);
    state.total = state.total + 1;
  },
  [types.DELETED](state, payload) {
    let idx = state.items.findIndex(p => p.groupId === payload.groupId);
    state.items.splice(idx, 1);
    state.total = state.total - 1;
    if (state.current && state.current.groupId == payload.groupId) {
      state.current = state.items.length > 0 ? state.items[0] : null;
    }
  },
  [types.UPDATED](state, payload) {
    let idx = state.items.findIndex(p => p.groupId === payload.groupId);
    state.items.splice(idx, 1, payload);
  },
  apps(state, datas) {
    state.apps = datas;
  },
  roles(state, payload) {
    state.roles = payload;
  },
  groupRoles(state, payload) {
    state.roleAll = payload.data;
    state.roletotal = payload.total;
  },
};
