import assert from 'assert';
import * as types from '../.types.js';
import _ from 'lodash';
import axios from 'axios';
import config from '@frame/config';
const { pageSize = 10 } = config;

const api = {
  users: '/xms/user/infos',
  deleteAll: '/xms/user/infos/delete',
  usersTow: '/xms/user/accts',
  orgUsers: '/xms/bff/org/users',
  groupUsers: '/xms/bff/group/users',
  // 绑定关系 source 部门id  target 用户id
  orgBind: '/xms/bind/org-user/items',
  // 应用帐号绑定关系 source appId  target acctId
  appBind: '/xms/bind/user-app/items',
  // 帐号角色绑定关系 source acctId  target roleId
  roleBind: '/xms/bind/user-role/items',
  apps: '/xms/bff/user/apps',
  roles: '/xms/bff/user/roles',
  upload: '/xms/task/upload',
  process: '/xms/task/process',
  derive: '/xms/task/export',
  certDownload: '/xms/util/server.cer',
  merge: '/xms/bff/user/merge',
};
// initial state
// shape: [{ id, quantity }]
export const state = () => ({
  items: [], //人员信息数据
  bindItems: [], //人员信息数据
  current: null,
  total: 0,
  apps: null,
  roles: [],
  user_Roles: [], // 用户所有角色绑定
});

// actions
export const actions = {
  // 批量删除
  async deleteAll({ commit }, payload) {
    const res = await this.$axios.$post(`${api.deleteAll}`, payload);
    commit('deleteAll', payload);
    return res;
  },
  async query({ commit }, { mode, orgId, groupId, filter = {}, skip = 0, limit = pageSize }) {
    let res;
    if (mode == 'org' /*orgId !== undefined && orgId !== null*/) {
      if (!orgId) {
        res = await this.$axios.$get(`${api.orgUsers}`, { params: { orgId, ...filter, skip, limit, 'sort.property': 'name' } });
      } else {
        res = await this.$axios.$get(`${api.orgUsers}`, { params: { orgId, ...filter, skip, limit, 'sort.property': 'user.name' } });
      }
    } else if (mode == 'group' /*groupId !== undefined && groupId !== null*/) {
      res = await this.$axios.$get(`${api.groupUsers}`, { params: { groupId, ...filter, skip, limit, 'sort.property': 'user.name' } });
    } else {
      res = await this.$axios.$get(`${api.users}`, { params: { ...filter, skip, limit } });
    }
    return res;
  },
  async load({ commit, dispatch }, { mode, orgId, groupId, filter = {}, paging = {} }) {
    const { page = 1, size = pageSize } = paging;
    const skip = Math.max(0, (page - 1) * size);
    const res = await dispatch('query', { mode, orgId, groupId, filter, skip, limit: size });
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.LOADED, res);
    }
    return res;
  },
  async create({ commit }, payload) {
    // create user info
    const res = await this.$axios.$post(`${api.users}`, payload);
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.CREATED, res);
    }
    return res;
  },
  // 合并用户
  async merge({ commit }, payload) {
    const res = await this.$axios.$post(`${api.merge}?userId=${payload[0].userId}&userId=${payload[1].userId}`);
    return res;
  },
  async createTow({ commit }, payload) {
    // create user info
    const res = await this.$axios.$post(`${api.usersTow}`, payload);
    // if (res.errcode === undefined || res.errcode === 0) {
    //   commit(types.CREATED, res);
    // }
    return res;
  },
  async delete({ commit, state }, { userId }) {
    const res = await this.$axios.$delete(`${api.users}/${userId}`);
    commit(types.DELETED, userId);
    return res;
  },
  async update({ commit, state, dispatch }, payload) {
    const { userId = state.current.userId } = payload;
    const res = await this.$axios.$post(`${api.users}/${userId}`, payload);
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.UPDATED, res);
    }
    return res;
  },
  async fetch({ commit }, payload = {}) {
    const { userId } = payload;
    if (!userId) {
      commit(types.DETAILS, {});
      return {};
    }

    // fetch user info
    const res = await this.$axios.$get(`${api.users}/${userId}`);
    if (res.errcode === undefined || res.errcode === 0) {
      commit(types.DETAILS, res);
    }
    return res;
  },
  async bindOrgs({ commit, state }, { orgIds, userId, replace = true } = {}) {
    // 同时绑定多个部门
    const res = await this.$axios.$post(`${api.orgBind}/batch`, { sources: orgIds, target: userId, userId }, { params: { replace } });
    return res;
  },
  async fetchOrgs({ commit, state }, { userId } = {}) {
    // 同时绑定多个部门
    const res = await this.$axios.$get(`${api.orgBind}`, { target: userId });
    return res;
  },
  // 添加用户应用绑定
  async bindApp({ commit, state, dispatch }, { userId, appId, mode = [], bindId } = {}) {
    let res;
    const param = mode.join(',');
    if (!bindId) {
      // 添加帐号和应用绑定关系
      res = await this.$axios.$post(`${api.appBind}`, { source: userId, target: appId, userId, appId, param });
    } else {
      // 修改帐号和应用绑定参数
      res = await this.$axios.$post(`${api.appBind}/${bindId}`, { param });
    }
    if (res.errcode === undefined || res.errcode === 0) {
      // 重新加载帐号app列表
      dispatch('fetchUserApps', { userId });
    }
    return res;
  },
  // 查询用户应用关联
  async fetchUserApps({ commit, dispatch }, { userId }) {
    if (!userId) {
      commit('apps', []); // clear
      return;
    }
    const res = await this.$axios.$get(`${api.apps}`, { userId });
    if (res.errcode === undefined || res.errcode === 0) {
      commit('apps', res.data);
    }
    return res;
  },
  // 删除部门应用绑定
  async deleteUserApp({ commit, state, dispatch }, { userId, appId }) {
    let res = await this.$axios.$delete(`${api.appBind}`, { source: userId, target: appId });
    if (res.errcode && res.errcode !== 0) {
      console.warn('删除用户-应用关系失败');
      return res;
    }
    if (res.errcode === undefined || res.errcode === 0) {
      // 重新加载帐号app列表
      dispatch('fetchUserApps', { userId, appId });
    }
    // 删除用户中该应用角色
    res = await this.$axios.$delete(`${api.roleBind}`, { appId, source: userId });
    return res;
  },
  // 查询用户应用  角色
  async fetchUserRoles({ commit, dispatch }, { userId, appId }) {
    if (!userId && appId) {
      commit('roles', []); // clear
      return;
    }
    const res = await this.$axios.$get(`${api.roleBind}`, { source: userId, appId });
    if (res.errcode === undefined || res.errcode === 0) {
      commit('roles', res.data);
    }
    return res;
  },
  // 用户所有角色绑定
  async userRoles({ commit, dispatch }, { userId }) {
    if (!userId) {
      commit('userRoles', []); // clear
      return;
    }
    const res = await this.$axios.$get(`${api.roleBind}`, { source: userId });
    if (res.errcode === undefined || res.errcode === 0) {
      commit('userRoles', res.data);
    }
    return res;
  },
  // 添加用户角色绑定关系
  async bindRoles({ commit, state, dispatch }, { appId, userId, roles } = {}) {
    const res = await this.$axios.$post(`${api.roleBind}/batch`, { source: userId, targets: roles, appId, userId });
    return res;
  },
  // 删除角色
  async deleteRoles({ commit, state }, { ids = [] }) {
    return Promise.all(ids.map(e => this.$axios.$delete(`${api.roleBind}/${e}`)));
  },
  // 文件上传
  async upload({ commit, state, dispatch }, payload) {
    const userId = this.getters.userinfo.userId;
    let param = new FormData();
    let config = { headers: { 'Content-Type': 'multipart/form-data' } };
    param.append('file', payload.file);
    const res = await this.$axios.$post(`${api.upload}/${payload.type}`, param, { userId }, config);
    return res;
  },
  // 进度查询
  async process({ commit, state, dispatch }, payload) {
    const res = await this.$axios.$get(`${api.process}/${payload}`);
    return res;
  },
  // 导出
  async derive({ commit, state, dispatch }, payload) {
    const userId = this.getters.userinfo.userId;
    const res = await this.$axios.$get(`${api.derive}/${payload.type}?userId=${userId}&typeId=${payload.typeId || null}`);
    return res;
    // 群组机构都是user  types是区分类型的  org 或。。。
  },
};

// mutations
export const mutations = {
  [types.LOADED](state, payload = {}) {
    state.items = payload.data;
    state.total = payload.total;
  },
  [types.CREATED](state, payload) {
    state.items.push(payload);
    state.total = state.total + 1;
    state.current = payload;
  },
  [types.DELETED](state, payload) {
    const idx = state.items.findIndex(p => p.userId === payload);
    state.items.splice(idx, 1);
    state.total = state.total - 1;
  },
  [types.UPDATED](state, payload) {
    const idx = state.items.findIndex(p => p.userId === payload.userId);
    state.items.splice(idx, 1, payload);
  },
  [types.DETAILS](state, payload) {
    state.current = payload;
  },
  [types.SELECTED](state, payload) {
    state.current = payload;
  },
  apps(state, datas) {
    state.apps = datas;
  },
  roles(state, payload) {
    state.roles = payload;
  },
  userRoles(state, payload) {
    state.user_Roles = payload;
  },
  deleteAll(state, payload) {
    state.items = state.items.filter(i => !payload.includes(i.userId));
  },
};
